import re
import unittest

import sklearn # noqa
from sklearn.datasets import fetch_20newsgroups
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.metrics import f1_score
from sklearn.naive_bayes import MultinomialNB
from sklearn.pipeline import make_pipeline

import numpy as np

from lime.lime_text import LimeTextExplainer
from lime.lime_text import IndexedCharacters, IndexedString


class TestLimeText(unittest.TestCase):

    def test_lime_text_explainer_good_regressor(self):
        categories = ['alt.atheism', 'soc.religion.christian']
        newsgroups_train = fetch_20newsgroups(subset='train',
                                              categories=categories)
        newsgroups_test = fetch_20newsgroups(subset='test',
                                             categories=categories)
        class_names = ['atheism', 'christian']
        vectorizer = TfidfVectorizer(lowercase=False)
        train_vectors = vectorizer.fit_transform(newsgroups_train.data)
        test_vectors = vectorizer.transform(newsgroups_test.data)
        nb = MultinomialNB(alpha=.01)
        nb.fit(train_vectors, newsgroups_train.target)
        pred = nb.predict(test_vectors)
        f1_score(newsgroups_test.target, pred, average='weighted')
        c = make_pipeline(vectorizer, nb)
        explainer = LimeTextExplainer(class_names=class_names)
        idx = 83
        exp = explainer.explain_instance(newsgroups_test.data[idx],
                                         c.predict_proba, num_features=6)
        self.assertIsNotNone(exp)
        self.assertEqual(6, len(exp.as_list()))

    def test_lime_text_tabular_equal_random_state(self):
        categories = ['alt.atheism', 'soc.religion.christian']
        newsgroups_train = fetch_20newsgroups(subset='train',
                                              categories=categories)
        newsgroups_test = fetch_20newsgroups(subset='test',
                                             categories=categories)
        class_names = ['atheism', 'christian']
        vectorizer = TfidfVectorizer(lowercase=False)
        train_vectors = vectorizer.fit_transform(newsgroups_train.data)
        test_vectors = vectorizer.transform(newsgroups_test.data)
        nb = MultinomialNB(alpha=.01)
        nb.fit(train_vectors, newsgroups_train.target)
        pred = nb.predict(test_vectors)
        f1_score(newsgroups_test.target, pred, average='weighted')
        c = make_pipeline(vectorizer, nb)

        explainer = LimeTextExplainer(class_names=class_names, random_state=10)
        exp_1 = explainer.explain_instance(newsgroups_test.data[83],
                                           c.predict_proba, num_features=6)

        explainer = LimeTextExplainer(class_names=class_names, random_state=10)
        exp_2 = explainer.explain_instance(newsgroups_test.data[83],
                                           c.predict_proba, num_features=6)

        self.assertTrue(exp_1.as_map() == exp_2.as_map())

    def test_lime_text_tabular_not_equal_random_state(self):
        categories = ['alt.atheism', 'soc.religion.christian']
        newsgroups_train = fetch_20newsgroups(subset='train',
                                              categories=categories)
        newsgroups_test = fetch_20newsgroups(subset='test',
                                             categories=categories)
        class_names = ['atheism', 'christian']
        vectorizer = TfidfVectorizer(lowercase=False)
        train_vectors = vectorizer.fit_transform(newsgroups_train.data)
        test_vectors = vectorizer.transform(newsgroups_test.data)
        nb = MultinomialNB(alpha=.01)
        nb.fit(train_vectors, newsgroups_train.target)
        pred = nb.predict(test_vectors)
        f1_score(newsgroups_test.target, pred, average='weighted')
        c = make_pipeline(vectorizer, nb)

        explainer = LimeTextExplainer(
            class_names=class_names, random_state=10)
        exp_1 = explainer.explain_instance(newsgroups_test.data[83],
                                           c.predict_proba, num_features=6)

        explainer = LimeTextExplainer(
            class_names=class_names, random_state=20)
        exp_2 = explainer.explain_instance(newsgroups_test.data[83],
                                           c.predict_proba, num_features=6)

        self.assertFalse(exp_1.as_map() == exp_2.as_map())

    def test_indexed_characters_bow(self):
        s = 'Please, take your time'
        inverse_vocab = ['P', 'l', 'e', 'a', 's', ',', ' ', 't', 'k', 'y', 'o', 'u', 'r', 'i', 'm']
        positions = [[0], [1], [2, 5, 11, 21], [3, 9],
                     [4], [6], [7, 12, 17], [8, 18], [10],
                     [13], [14], [15], [16], [19], [20]]
        ic = IndexedCharacters(s)

        self.assertTrue(np.array_equal(ic.as_np, np.array(list(s))))
        self.assertTrue(np.array_equal(ic.string_start, np.arange(len(s))))
        self.assertTrue(ic.inverse_vocab == inverse_vocab)
        self.assertTrue(ic.positions == positions)

    def test_indexed_characters_not_bow(self):
        s = 'Please, take your time'

        ic = IndexedCharacters(s, bow=False)

        self.assertTrue(np.array_equal(ic.as_np, np.array(list(s))))
        self.assertTrue(np.array_equal(ic.string_start, np.arange(len(s))))
        self.assertTrue(ic.inverse_vocab == list(s))
        self.assertTrue(np.array_equal(ic.positions, np.arange(len(s))))

    def test_indexed_string_regex(self):
        s = 'Please, take your time. Please'
        tokenized_string = np.array(
            ['Please', ', ', 'take', ' ', 'your', ' ', 'time', '. ', 'Please'])
        inverse_vocab = ['Please', 'take', 'your', 'time']
        start_positions = [0, 6, 8, 12, 13, 17, 18, 22, 24]
        positions = [[0, 8], [2], [4], [6]]
        indexed_string = IndexedString(s)

        self.assertTrue(np.array_equal(indexed_string.as_np, tokenized_string))
        self.assertTrue(np.array_equal(indexed_string.string_start, start_positions))
        self.assertTrue(indexed_string.inverse_vocab == inverse_vocab)
        self.assertTrue(np.array_equal(indexed_string.positions, positions))

    def test_indexed_string_callable(self):
        s = 'aabbccddaa'

        def tokenizer(string):
            return [string[i] + string[i + 1] for i in range(0, len(string) - 1, 2)]

        tokenized_string = np.array(['aa', 'bb', 'cc', 'dd', 'aa'])
        inverse_vocab = ['aa', 'bb', 'cc', 'dd']
        start_positions = [0, 2, 4, 6, 8]
        positions = [[0, 4], [1], [2], [3]]
        indexed_string = IndexedString(s, tokenizer)

        self.assertTrue(np.array_equal(indexed_string.as_np, tokenized_string))
        self.assertTrue(np.array_equal(indexed_string.string_start, start_positions))
        self.assertTrue(indexed_string.inverse_vocab == inverse_vocab)
        self.assertTrue(np.array_equal(indexed_string.positions, positions))

    def test_indexed_string_inverse_removing_tokenizer(self):
        s = 'This is a good movie. This, it is a great movie.'

        def tokenizer(string):
            return re.split(r'(?:\W+)|$', string)

        indexed_string = IndexedString(s, tokenizer)

        self.assertEqual(s, indexed_string.inverse_removing([]))

    def test_indexed_string_inverse_removing_regex(self):
        s = 'This is a good movie. This is a great movie'
        indexed_string = IndexedString(s)

        self.assertEqual(s, indexed_string.inverse_removing([]))


if __name__ == '__main__':
    unittest.main()



import unittest
from lime.wrappers.scikit_image import BaseWrapper
from lime.wrappers.scikit_image import SegmentationAlgorithm
from skimage.segmentation import quickshift
from skimage.data import chelsea
from skimage.util import img_as_float
import numpy as np


class TestBaseWrapper(unittest.TestCase):

    def test_base_wrapper(self):

        obj_with_params = BaseWrapper(a=10, b='message')
        obj_without_params = BaseWrapper()

        def foo_fn():
            return 'bar'

        obj_with_fn = BaseWrapper(foo_fn)
        self.assertEqual(obj_with_params.target_params, {'a': 10, 'b': 'message'})
        self.assertEqual(obj_without_params.target_params, {})
        self.assertEqual(obj_with_fn.target_fn(), 'bar')

    def test__check_params(self):

        def bar_fn(a):
            return str(a)

        class Pipo():

            def __init__(self):
                self.name = 'pipo'

            def __call__(self, message):
                return message

        pipo = Pipo()
        obj_with_valid_fn = BaseWrapper(bar_fn, a=10, b='message')
        obj_with_valid_callable_fn = BaseWrapper(pipo, c=10, d='message')
        obj_with_invalid_fn = BaseWrapper([1, 2, 3], fn_name='invalid')

        # target_fn is not a callable or function/method
        with self.assertRaises(AttributeError):
            obj_with_invalid_fn._check_params('fn_name')

        # parameters is not in target_fn args
        with self.assertRaises(ValueError):
            obj_with_valid_fn._check_params(['c'])
            obj_with_valid_callable_fn._check_params(['e'])

        # params is in target_fn args
        try:
            obj_with_valid_fn._check_params(['a'])
            obj_with_valid_callable_fn._check_params(['message'])
        except Exception:
            self.fail("_check_params() raised an unexpected exception")

        # params is not a dict or list
        with self.assertRaises(TypeError):
            obj_with_valid_fn._check_params(None)
        with self.assertRaises(TypeError):
            obj_with_valid_fn._check_params('param_name')

    def test_set_params(self):

        class Pipo():

            def __init__(self):
                self.name = 'pipo'

            def __call__(self, message):
                return message
        pipo = Pipo()
        obj = BaseWrapper(pipo)

        # argument is set accordingly
        obj.set_params(message='OK')
        self.assertEqual(obj.target_params, {'message': 'OK'})
        self.assertEqual(obj.target_fn(**obj.target_params), 'OK')

        # invalid argument is passed
        try:
            obj = BaseWrapper(Pipo())
            obj.set_params(invalid='KO')
        except Exception:
            self.assertEqual(obj.target_params, {})

    def test_filter_params(self):

        # right arguments are kept and wrong dismmissed
        def baz_fn(a, b, c=True):
            if c:
                return a + b
            else:
                return a
        obj_ = BaseWrapper(baz_fn, a=10, b=100, d=1000)
        self.assertEqual(obj_.filter_params(baz_fn), {'a': 10, 'b': 100})

        # target_params is overriden using 'override' argument
        self.assertEqual(obj_.filter_params(baz_fn, override={'c': False}),
                         {'a': 10, 'b': 100, 'c': False})


class TestSegmentationAlgorithm(unittest.TestCase):

    def test_instanciate_segmentation_algorithm(self):
        img = img_as_float(chelsea()[::2, ::2])

        # wrapped functions provide the same result
        fn = SegmentationAlgorithm('quickshift', kernel_size=3, max_dist=6,
                                   ratio=0.5, random_seed=133)
        fn_result = fn(img)
        original_result = quickshift(img, kernel_size=3, max_dist=6, ratio=0.5,
                                     random_seed=133)

        # same segments
        self.assertTrue(np.array_equal(fn_result, original_result))

    def test_instanciate_slic(self):
        pass

    def test_instanciate_felzenszwalb(self):
        pass


if __name__ == '__main__':
    unittest.main()



import unittest
from unittest import TestCase

import numpy as np

from sklearn.datasets import load_iris

from lime.discretize import QuartileDiscretizer, DecileDiscretizer, EntropyDiscretizer


class TestDiscretize(TestCase):

    def setUp(self):
        iris = load_iris()

        self.feature_names = iris.feature_names
        self.x = iris.data
        self.y = iris.target

    def check_random_state_for_discretizer_class(self, DiscretizerClass):
        # ----------------------------------------------------------------------
        # -----------Check if the same random_state produces the same-----------
        # -------------results for different discretizer instances.-------------
        # ----------------------------------------------------------------------
        discretizer = DiscretizerClass(self.x, [], self.feature_names, self.y,
                                       random_state=10)
        x_1 = discretizer.undiscretize(discretizer.discretize(self.x))

        discretizer = DiscretizerClass(self.x, [], self.feature_names, self.y,
                                       random_state=10)
        x_2 = discretizer.undiscretize(discretizer.discretize(self.x))

        self.assertEqual((x_1 == x_2).sum(), x_1.shape[0] * x_1.shape[1])

        discretizer = DiscretizerClass(self.x, [], self.feature_names, self.y,
                                       random_state=np.random.RandomState(10))
        x_1 = discretizer.undiscretize(discretizer.discretize(self.x))

        discretizer = DiscretizerClass(self.x, [], self.feature_names, self.y,
                                       random_state=np.random.RandomState(10))
        x_2 = discretizer.undiscretize(discretizer.discretize(self.x))

        self.assertEqual((x_1 == x_2).sum(), x_1.shape[0] * x_1.shape[1])

        # ----------------------------------------------------------------------
        # ---------Check if two different random_state values produces----------
        # -------different results for different discretizers instances.--------
        # ----------------------------------------------------------------------
        discretizer = DiscretizerClass(self.x, [], self.feature_names, self.y,
                                       random_state=10)
        x_1 = discretizer.undiscretize(discretizer.discretize(self.x))

        discretizer = DiscretizerClass(self.x, [], self.feature_names, self.y,
                                       random_state=20)
        x_2 = discretizer.undiscretize(discretizer.discretize(self.x))

        self.assertFalse((x_1 == x_2).sum() == x_1.shape[0] * x_1.shape[1])

        discretizer = DiscretizerClass(self.x, [], self.feature_names, self.y,
                                       random_state=np.random.RandomState(10))
        x_1 = discretizer.undiscretize(discretizer.discretize(self.x))

        discretizer = DiscretizerClass(self.x, [], self.feature_names, self.y,
                                       random_state=np.random.RandomState(20))
        x_2 = discretizer.undiscretize(discretizer.discretize(self.x))

        self.assertFalse((x_1 == x_2).sum() == x_1.shape[0] * x_1.shape[1])

    def test_random_state(self):
        self.check_random_state_for_discretizer_class(QuartileDiscretizer)

        self.check_random_state_for_discretizer_class(DecileDiscretizer)

        self.check_random_state_for_discretizer_class(EntropyDiscretizer)

    def test_feature_names_1(self):
        self.maxDiff = None
        discretizer = QuartileDiscretizer(self.x, [], self.feature_names,
                                          self.y, random_state=10)
        self.assertDictEqual(
            {0: ['sepal length (cm) <= 5.10',
                 '5.10 < sepal length (cm) <= 5.80',
                 '5.80 < sepal length (cm) <= 6.40',
                 'sepal length (cm) > 6.40'],
             1: ['sepal width (cm) <= 2.80',
                 '2.80 < sepal width (cm) <= 3.00',
                 '3.00 < sepal width (cm) <= 3.30',
                 'sepal width (cm) > 3.30'],
             2: ['petal length (cm) <= 1.60',
                 '1.60 < petal length (cm) <= 4.35',
                 '4.35 < petal length (cm) <= 5.10',
                 'petal length (cm) > 5.10'],
             3: ['petal width (cm) <= 0.30',
                 '0.30 < petal width (cm) <= 1.30',
                 '1.30 < petal width (cm) <= 1.80',
                 'petal width (cm) > 1.80']},
            discretizer.names)

    def test_feature_names_2(self):
        self.maxDiff = None
        discretizer = DecileDiscretizer(self.x, [], self.feature_names, self.y,
                                        random_state=10)
        self.assertDictEqual(
            {0: ['sepal length (cm) <= 4.80',
                 '4.80 < sepal length (cm) <= 5.00',
                 '5.00 < sepal length (cm) <= 5.27',
                 '5.27 < sepal length (cm) <= 5.60',
                 '5.60 < sepal length (cm) <= 5.80',
                 '5.80 < sepal length (cm) <= 6.10',
                 '6.10 < sepal length (cm) <= 6.30',
                 '6.30 < sepal length (cm) <= 6.52',
                 '6.52 < sepal length (cm) <= 6.90',
                 'sepal length (cm) > 6.90'],
             1: ['sepal width (cm) <= 2.50',
                 '2.50 < sepal width (cm) <= 2.70',
                 '2.70 < sepal width (cm) <= 2.80',
                 '2.80 < sepal width (cm) <= 3.00',
                 '3.00 < sepal width (cm) <= 3.10',
                 '3.10 < sepal width (cm) <= 3.20',
                 '3.20 < sepal width (cm) <= 3.40',
                 '3.40 < sepal width (cm) <= 3.61',
                 'sepal width (cm) > 3.61'],
             2: ['petal length (cm) <= 1.40',
                 '1.40 < petal length (cm) <= 1.50',
                 '1.50 < petal length (cm) <= 1.70',
                 '1.70 < petal length (cm) <= 3.90',
                 '3.90 < petal length (cm) <= 4.35',
                 '4.35 < petal length (cm) <= 4.64',
                 '4.64 < petal length (cm) <= 5.00',
                 '5.00 < petal length (cm) <= 5.32',
                 '5.32 < petal length (cm) <= 5.80',
                 'petal length (cm) > 5.80'],
             3: ['petal width (cm) <= 0.20',
                 '0.20 < petal width (cm) <= 0.40',
                 '0.40 < petal width (cm) <= 1.16',
                 '1.16 < petal width (cm) <= 1.30',
                 '1.30 < petal width (cm) <= 1.50',
                 '1.50 < petal width (cm) <= 1.80',
                 '1.80 < petal width (cm) <= 1.90',
                 '1.90 < petal width (cm) <= 2.20',
                 'petal width (cm) > 2.20']},
            discretizer.names)

    def test_feature_names_3(self):
        self.maxDiff = None
        discretizer = EntropyDiscretizer(self.x, [], self.feature_names,
                                         self.y, random_state=10)
        self.assertDictEqual(
            {0: ['sepal length (cm) <= 4.85',
                 '4.85 < sepal length (cm) <= 5.45',
                 '5.45 < sepal length (cm) <= 5.55',
                 '5.55 < sepal length (cm) <= 5.85',
                 '5.85 < sepal length (cm) <= 6.15',
                 '6.15 < sepal length (cm) <= 7.05',
                 'sepal length (cm) > 7.05'],
             1: ['sepal width (cm) <= 2.45',
                 '2.45 < sepal width (cm) <= 2.95',
                 '2.95 < sepal width (cm) <= 3.05',
                 '3.05 < sepal width (cm) <= 3.35',
                 '3.35 < sepal width (cm) <= 3.45',
                 '3.45 < sepal width (cm) <= 3.55',
                 'sepal width (cm) > 3.55'],
             2: ['petal length (cm) <= 2.45',
                 '2.45 < petal length (cm) <= 4.45',
                 '4.45 < petal length (cm) <= 4.75',
                 '4.75 < petal length (cm) <= 5.15',
                 'petal length (cm) > 5.15'],
             3: ['petal width (cm) <= 0.80',
                 '0.80 < petal width (cm) <= 1.35',
                 '1.35 < petal width (cm) <= 1.75',
                 '1.75 < petal width (cm) <= 1.85',
                 'petal width (cm) > 1.85']},
            discretizer.names)


if __name__ == '__main__':
    unittest.main()
